import argparse, os
from . import scaffolds

def parse_command(command):
	if command.command == 'new':
		root_path = os.path.abspath(os.getcwd())
		if command.template:
			scaffold = getattr(scaffolds, ucfirst(command.template))
			if not scaffold:
				raise RuntimeError('Specified scaffold not found.')
		else:
			scaffold = scaffolds.Default
		s = scaffold(command.name)
		s.write(root_path)

def main():
	parser = argparse.ArgumentParser(description='The CLI interface for managing your pain.')
	subs = parser.add_subparsers(dest='command')
	new = subs.add_parser('new')
	new.add_argument('name')
	new.add_argument('-t', '--template')
	new.add_argument('-c', '--config', help='Specify a config file to read in default information.')
	cli_input = parser.parse_args()
	return parse_command(cli_input)


import string
''' Abstract File Class '''

class File(object):

	def __init__(self, name):
		self.name = name
		self.content = ''

	def template(self, values):
		return string.Template(self.content).safe_substitute(**values)

class SetupPy(File):

	def __init__(self):
		super(SetupPy, self).__init__('setup.py')
		self.content = '''
			from setuptools import setup, find_packages
			
			setup(
				name='$name',
				version='0.1',
				description='$description',
				classifiers=[],
				keywords='',
				author='$author',
				author_email='$author_email',
				url='$url',
				license='$license',
				packages=find_packages(),
				include_package_data=True,
				test_suite='nose.collector',
				zip_safe=False,
			)
		'''
class SetupCfg(File):

	def __init__(self):
		super(SetupCfg, self).__init__('setup.cfg')
		self.content = ''

class Requirements(File):

	def __init__(self):
		super(Requirements, self).__init__('requirements.txt')
		self.content = ''

class TODO(File):

	def __init__(self):
		super(TODO, self).__init__('TODO.md')
		self.content = ''

class README(File):

	def __init__(self):
		super(README, self).__init__('README.md')
		self.content = ''

class MANIFEST(File):

	def __init__(self):
		super(MANIFEST, self).__init__('MANIFEST.in')
		self.content = ''

class TravisYaml(File):

	def __init__(self):
		super(TravisYaml, self).__init__('.travis-yml')
		self.content = ''

class GitIgnore(File):

	def __init__(self):
		super(GitIgnore, self).__init__('.gitignore')
		self.content = '''
			*.py[cod]
			.env
			# C extensions
			*.so

			# Packages
			*.egg
			*.egg-info
			dist
			build
			eggs
			parts
			bin
			var
			sdist
			develop-eggs
			.installed.cfg
			lib
			lib64

			# Installer logs
			pip-log.txt

			# Unit test / coverage reports
			.coverage
			.tox
			nosetests.xml

			# Translations
			*.mo

			# Mr Developer
			.mr.developer.cfg
			.project
			.pydevproject
		'''
class Env(File):

	def __init__(self):
		super(Env, self).__init__('.env')
		self.content = '''
			use_env $name
		'''

class Init(File):

	def __init__(self):
		super(Init, self).__init__('__init__.py')
		self.content = ''


import os, string
from .files import *

''' Abstract scaffold class '''

class Scaffold(object):

	def __init__(self, name, **config):
		self.config = config
		self.config.update(name=name)
		self.config.update(is_package=True)
		self.files = {}

	def add_file(self, path, f):
		if path not in self.files:
			self.files[path] = []
		self.files[path].append(f)

	def write(self, root_dir):
		if self.config.get('is_package'):
			root_dir = os.path.join(root_dir, self.config.get('name'))
		for path, files in self.files.iteritems():
				for f in files:
					write_path = '%s%s' % (root_dir, os.path.join(string.Template(path).safe_substitute(**self.config), f.name))
					if not os.path.exists(os.path.dirname(write_path)):
						os.makedirs(os.path.dirname(write_path))
					text = f.template(self.config)
					with open(write_path, 'w+') as f:
						f.write(text)

class Default(Scaffold):

	def __init__(self, name, **config):
		super(Default, self).__init__(name, **config)

		self.files = {
			'/': [SetupPy(), SetupCfg(), Requirements(), TODO(), README(), MANIFEST(), TravisYaml(), GitIgnore(), Env()],
			'/$name': [Init()],
			'/$name/tests': [Init()]
		}


from setuptools import setup, find_packages

setup(name='pain',
      version='0.1',
      description='A scaffolding tool to mitigate the pain of setting up a new project.',
      long_description=__doc__,
      classifiers=[
            'Development Status :: 3 - Alpha',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'Programming Language :: Python'
      ],
      keywords='scaffolding scaffold python setup project new',
      author='Peter M. Elias',
      author_email='petermelias@gmail.com',
      url='http://github.com/petermelias/pain',
      license='FreeBSD',
      platforms='any',
      packages=find_packages(),
      include_package_data=True,
      test_suite = "nose.collector",
      zip_safe=False,
      entry_points = {
        'console_scripts': [ 'pain = pain:main' ]
      }
)